from abc import ABC, abstractmethod
from typing import List
import pandas as pd
import plotly.express as px
from pathlib import Path

import dqfit as dq

from dqfit.dimensions import Conformant, Complete, Plausible
from dqfit.transform import transform_to_fhir_path


class DQIBase(ABC):
    def __init__(
        self,
        context_key: str,
        Dimensions: list = [Conformant],
    ) -> None:
        self.context_key = context_key
        self.context = dq.io.load_context(context_key)
        self.Dimensions = Dimensions  # Model Dimensions
        self.M = Dimensions  # Model Dimensions
        self.m = pd.DataFrame(self.context["dim"])  # Context Dimensions
        # self.path_level_result = self.fit()
        # self.context_fitness = self._get_context_fitness()
        # self.index = self._get_index()
        # self.patient_refs = self.path_result["_ref"].unique()
        # print(self.shape)

    def __repr__(self) -> str:
        return self.__class__.__name__

    @property
    def M_KEYS(self):
        return [dim.__name__ for dim in self.M]

    @property
    def shape(self):
        """
        (Patient x Resource x Path) x (Context Dimensions x Model Dimensions)
        ::
        (organism x molecule x atom) x (context x model)
        """
        return (
            (
                self.fhir_path["patient_id"].nunique(),
                self.fhir_path["id"].nunique(),
                len(self.fhir_path),
                len(self.M),
            ),
            (len(self.m), len(self.M), 1),  # weight
        )

    def fit(self, fhir_resources: List[dict]):
        """
        Emulates sklearn fit method.
        1. Takes in a list of FHIR resources
        2. Transforms into FHIR Path
        3. Fits each Model Dimension
        4. Aggregates Patient Level Scores
        5. Returns Results DataFrame of shape (m, M)
        """
        fhir_path = transform_to_fhir_path(fhir_resources)
        for Dim in self.Dimensions:
            fhir_path = Dim.fit(fhir_path, self.context)
        fhir_path = fhir_path.query("Conformant == 1").reset_index(drop=True)
        self.fhir_path = fhir_path  # for patient level results

        patient_ids = fhir_path["patient_id"].unique()
        patient_level = [
            self.get_patient_level_result(patient_id) for patient_id in patient_ids
        ]
        result = (
            pd.concat(patient_level)
            .groupby("path")
            .agg({dim.__name__: "mean" for dim in self.Dimensions})
            .reset_index()
            .merge(self.m, how="right", left_on="path", right_on="path")
        )
        result["Score"] = result[[*self.M_KEYS, "W"]].product(axis=1)
        self.result = result
        self.index = round(result["Score"].sum(), 1)
        return result

    def get_patient_level_result(self, patient_id: str):
        assert self.fhir_path is not None
        patient_level_result = (
            (
                self.fhir_path.query(f"patient_id == '{patient_id}'")
                .groupby("path")
                .agg({dim.__name__: "mean" for dim in self.Dimensions})
            )
            .reset_index()
            .merge(self.m, how="right", left_on="path", right_on="path")
            .fillna(0)
        )
        patient_level_result.insert(0, "patient_id", patient_id)
        return patient_level_result

    def visualize(self):
        df = self.result.copy()
        df["resourceType"] = df["path"].apply(lambda x: x.split(".")[0])
        return px.scatter(
            df.melt(
                id_vars=["path", "W", "resourceType"],
                var_name="Dimension",
                value_vars=self.M_KEYS,
                value_name="score",
            ).sort_values("path", ascending=False),
            x="score",
            y="path",
            facet_col="Dimension",
            color="resourceType",
            size="W",
            title=f"{self} | {self.context_key} | Index: {self.index} <sup><br>{self.shape} | (Patient x Resource x Path), (ContextDimension, ModelDimension, W)",
            height=600,
        )


class DQI2(DQIBase):
    def __init__(
        self,
        context_key: str,
        Dimensions: list = [Conformant, Complete],
    ) -> None:
        super().__init__(context_key, Dimensions)


class DQI3(DQIBase):
    def __init__(
        self,
        context_key: str,
        Dimensions: list = [Conformant, Complete, Plausible],
    ) -> None:
        super().__init__(context_key, Dimensions)
